#ifndef _MYQUE_H
#define _MYQUE_H
/**********************************************************************************************/
/*
QUE is a generic template class which implements a simple first in last out que and can store
any arbitrary class which supports the copy operator and has a Next pointer.

When a message is stored memory is dynamically allocated. When you Get a message from
the QUE it is removed, and if non-NULL it is the responsibility of the thread which
retrieved the message to delete it.

//This class not currently exported from i2cmgr, for now I've duplicated it here.
//I suppose you can't really export a template class, but would have
//to include the implementation in the header file like I've done here
*/

template <class MSG>
class MYQUE
{
public:
   MYQUE();
   ~MYQUE();
   HANDLE hReady;
   void Store(MSG* Msg);                                // Stores a message in the Que
   void Store(const MSG& Msg) { Store(new MSG(Msg)); }; // Makes a copy of a message and stores it
   MSG* Get();                                          // Removes a message from the Que, delete it when done
   void Flush();
private:
   CRITICAL_SECTION QueLock;
   MSG* List;
};


/**********************************************************************************************/

/********************************************************************************************/

template <class MSG>
   MYQUE<MSG>::MYQUE()
{
   hReady = CreateEvent(0, FALSE, FALSE, NULL); // Auto clear and initially cleared
   InitializeCriticalSection(&QueLock);
   List = NULL;
};

template <class MSG>
   MYQUE<MSG>::~MYQUE()
{
   Flush();
   CloseHandle(hReady);
   DeleteCriticalSection(&QueLock);
};

template <class MSG>
   void MYQUE<MSG>::Flush()
{
   MSG* Msg;
   EnterCriticalSection(&QueLock);
   while (Msg = List)
   {
      List = List->Next;
      delete Msg;
   };
   ResetEvent(hReady);
   LeaveCriticalSection(&QueLock);
};


template <class MSG>
   void MYQUE<MSG>::Store(MSG* Msg)
{
   Msg->Next = NULL;

   MSG** Tail = &List;

   EnterCriticalSection(&QueLock);

   while (*Tail)
      Tail = &((*Tail)->Next);

   *Tail = Msg;

   SetEvent(hReady);

   LeaveCriticalSection(&QueLock);
};


template <class MSG>
   MSG* MYQUE<MSG>::Get()
{
   MSG* Result;
   EnterCriticalSection(&QueLock);

   Result = List;
   if (List)
      List = List->Next;

   if (List)
      SetEvent(hReady);

   LeaveCriticalSection(&QueLock);
   return Result;
};

/********************************************************************************************/


#endif
